//------------------------------------------------------------------
// Random walk index developed by Micheal Poulos
// source : TASC 10:9 (385-389): Are There Persistent Cycles?
//------------------------------------------------------------------
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1  Green
#property indicator_color2  Red
#property indicator_color3  DarkGray
#property indicator_color4  DarkGray
#property indicator_style3  STYLE_DOT
#property indicator_width1  2
#property indicator_width2  2
#property indicator_minimum 0

//
//
//
//
//

extern int    RWILength        = 25;
extern int    T3Period         = 3;
extern double T3Hot            = 0.7;
extern bool   T3Original       = false;
extern int    T3SignalPeriod   = 50;
extern double T3SignalHot      = 0.7;
extern bool   T3SignalOriginal = false;

double RWIUp[];
double RWIDown[];
double Zline[];
double signal[];
double ranges[];

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   SetIndexBuffer(0,RWIUp);
   SetIndexBuffer(1,RWIDown);
   SetIndexBuffer(2,Zline);
   SetIndexBuffer(3,signal); 
   IndicatorShortName("Random walk index ("+RWILength+")");
   return(0);
}
int deinit()
{
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int start()
{
   int i,r,counted_bars=IndicatorCounted();
      if(counted_bars < 0) return(-1);
      if(counted_bars > 0) counted_bars--;
           int limit = MathMin(Bars-counted_bars,Bars-1);
           if (ArrayRange(ranges,0)!=Bars) ArrayResize(ranges,Bars);

   //
   //
   //
   //
   //
            
   for (i=limit, r=Bars-i-1; i>=0; i--,r++)
   {  
      ranges[r] = MathMax(High[i+1],Close[i+2])-MathMin(Low[i+1],Close[i+2]);
         double rwiUp = 0;
         double rwiDo = 0;
         double atr   = ranges[r];
         
         for (int k = 1; k < RWILength; k++)
         {
            atr += ranges[r-k];  
            
            //
            //
            //
            //
            //
            
            double denominator  = (atr/(k+1.0))*MathSqrt(k+1);
               if (denominator != 0)
               {
                  rwiUp = MathMax(rwiUp,(High[i] - Low[i+k]) / denominator);
                  rwiDo = MathMax(rwiDo,(High[i+k] - Low[i]) / denominator);
               }
         }
         RWIUp[i]   = iT3(rwiUp,T3Period,T3Hot,T3Original,i,0);
         RWIDown[i] = iT3(rwiDo,T3Period,T3Hot,T3Original,i,1);
         Zline[i]   = 1.00;
         if (T3SignalPeriod>1)
         if (RWIUp[i]>RWIDown[i])
               signal[i] = iT3(RWIUp[i]  ,T3SignalPeriod,T3SignalHot,T3SignalOriginal,i,2);
         else  signal[i] = iT3(RWIDown[i],T3SignalPeriod,T3SignalHot,T3SignalOriginal,i,2);
   }         
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double workT3[][18];
double workT3Coeffs[][6];
#define _period 0
#define _c1     1
#define _c2     2
#define _c3     3
#define _c4     4
#define _alpha  5

//
//
//
//
//

double iT3(double price, double period, double hot, bool original, int i, int instanceNo=0)
{
   if (ArrayRange(workT3,0) != Bars)                ArrayResize(workT3,Bars);
   if (ArrayRange(workT3Coeffs,0) < (instanceNo+1)) ArrayResize(workT3Coeffs,instanceNo+1);

   if (workT3Coeffs[instanceNo][_period] != period)
   {
     workT3Coeffs[instanceNo][_period] = period;
        double a = hot;
            workT3Coeffs[instanceNo][_c1] = -a*a*a;
            workT3Coeffs[instanceNo][_c2] = 3*a*a+3*a*a*a;
            workT3Coeffs[instanceNo][_c3] = -6*a*a-3*a-3*a*a*a;
            workT3Coeffs[instanceNo][_c4] = 1+3*a+a*a*a+3*a*a;
            if (original)
                 workT3Coeffs[instanceNo][_alpha] = 2.0/(1.0 + period);
            else workT3Coeffs[instanceNo][_alpha] = 2.0/(2.0 + (period-1.0)/2.0);
   }
   
   //
   //
   //
   //
   //
   
   int buffer = instanceNo*6;
   int r = Bars-i-1;
   if (r == 0)
      {
         workT3[r][0+buffer] = price;
         workT3[r][1+buffer] = price;
         workT3[r][2+buffer] = price;
         workT3[r][3+buffer] = price;
         workT3[r][4+buffer] = price;
         workT3[r][5+buffer] = price;
      }
   else
      {
         workT3[r][0+buffer] = workT3[r-1][0+buffer]+workT3Coeffs[instanceNo][_alpha]*(price              -workT3[r-1][0+buffer]);
         workT3[r][1+buffer] = workT3[r-1][1+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][0+buffer]-workT3[r-1][1+buffer]);
         workT3[r][2+buffer] = workT3[r-1][2+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][1+buffer]-workT3[r-1][2+buffer]);
         workT3[r][3+buffer] = workT3[r-1][3+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][2+buffer]-workT3[r-1][3+buffer]);
         workT3[r][4+buffer] = workT3[r-1][4+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][3+buffer]-workT3[r-1][4+buffer]);
         workT3[r][5+buffer] = workT3[r-1][5+buffer]+workT3Coeffs[instanceNo][_alpha]*(workT3[r][4+buffer]-workT3[r-1][5+buffer]);
      }

   //
   //
   //
   //
   //
   
   return(workT3Coeffs[instanceNo][_c1]*workT3[r][5+buffer] + 
          workT3Coeffs[instanceNo][_c2]*workT3[r][4+buffer] + 
          workT3Coeffs[instanceNo][_c3]*workT3[r][3+buffer] + 
          workT3Coeffs[instanceNo][_c4]*workT3[r][2+buffer]);
}